import unittest
from jgcmlib.module import Module

class TestModule(unittest.TestCase):
    def test_something(self):
        # Add your test case here
        pass

if __name__ == '__main__':
    unittest.main()